==> paragraph_blocks-4.1.1.tar.gz: paragraph_blocks/src/ParagraphBlocksLabeller.php <==
<?php

namespace Drupal\paragraph_blocks;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\paragraph_blocks\Entity\ParagraphBlocksEntity;

/**
 * Labels the paragraph blocks once the entity context is known.
 */
class ParagraphBlocksLabeller {

  use StringTranslationTrait;

  /**
   * The plugin type id.
   *
   * @var string
   */
  const PLUGIN_TYPE_ID = 'paragraph_field';

  /**
   * The label format.
   *
   * @var string
   */
  const LABEL_FORMAT = 'Page: @label';

  /**
   * The current entity, or NULL.
   *
   * @var \Drupal\Core\Entity\EntityInterface|null
   */
  protected ?EntityInterface $entity;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The paragraph blocks entity manager.
   *
   * @var \Drupal\paragraph_blocks\ParagraphBlocksEntityManager
   */
  protected ParagraphBlocksEntityManager $paragraphBlocksEntityManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * Constructs a new ParagraphBlocksLabeller object.
   *
   * @param \Drupal\paragraph_blocks\ParagraphBlocksEntityManager $paragraph_blocks_entity_manager
   *   The Paragraph blocks entity manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The interface for an entity field manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(ParagraphBlocksEntityManager $paragraph_blocks_entity_manager, EntityFieldManagerInterface $entity_field_manager, ModuleHandlerInterface $module_handler) {
    $this->paragraphBlocksEntityManager = $paragraph_blocks_entity_manager;
    $this->entity = $this->paragraphBlocksEntityManager->getRefererEntity();
    $this->entityFieldManager = $entity_field_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * Removed unused paragraphs and update the layout builder title.
   *
   * @param array $definitions
   *   The plugin definitions.
   */
  public function hookLayoutBuilderChooseBlocksAlter(array &$definitions) {
    // Loop through all the plugin definitions.
    foreach ($definitions as $plugin_id => $definition) {
      if ($this->isParagraphField($plugin_id)) {
        // Remove if this paragraph field is not enabled or if there is no paragraph data for the delta.
        if (!$this->paragraphFieldIsEnabled($plugin_id) || is_null($this->getParagraph($plugin_id))) {
          unset($definitions[$plugin_id]);
        }
        else {
          // Replace the title.
          $definitions[$plugin_id]['admin_label'] = $this->getTitle($plugin_id);
        }
      }
    }
  }

  /**
   * Check if the given plugin id represents a paragraph field.
   *
   * @param string $plugin_id
   *
   * @return bool
   */
  private function isParagraphField(string $plugin_id): bool {
    $plugin_parts = $this->getPluginParts($plugin_id);
    return $plugin_parts['plugin_type_id'] === self::PLUGIN_TYPE_ID;
  }

  /**
   * Check if paragraph field is enabled.
   *
   * Only check the field bundle if it exists. This is new to the 2.x branch.
   * So this check exists for backwards compatability with plugins saved using
   * the 1.x branch.
   *
   * @param string $plugin_id
   *
   * @return bool
   */
  private function paragraphFieldIsEnabled(string $plugin_id): bool {
    $plugin_parts = $this->getPluginParts($plugin_id);
    if (!empty($plugin_parts) && isset($plugin_parts['plugin_field_bundle'])) {
      $field_definitions = $this->entityFieldManager
        ->getFieldDefinitions($plugin_parts['plugin_entity_type_id'], $plugin_parts['plugin_field_bundle']);
      $field_config = $field_definitions[$plugin_parts['plugin_field_name']]
        ->getConfig($plugin_parts['plugin_field_bundle']);
      if ($field_config->getThirdPartySetting('paragraph_blocks', 'status', TRUE)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Returns the plugin's paragraph.
   *
   * @param string $plugin_id
   *   The plugin id.
   *
   * @return ?\Drupal\paragraph_blocks\Entity\ParagraphBlocksEntity
   *   The paragraph title.
   */
  public function getParagraph(string $plugin_id): ?ParagraphBlocksEntity {
    $plugin_parts = $this->getPluginParts($plugin_id);

    // Return NULL if this plugin id represents no valid paragraph data.
    if (!$this->entity
      || $this->entity->bundle() !== $plugin_parts['plugin_field_bundle']
      || $plugin_parts['plugin_field_delta'] >= $this->entity->get($plugin_parts['plugin_field_name'])->count()
    ) {
      return NULL;
    }

    // Get the referenced paragraph.
    return $this->entity->get($plugin_parts['plugin_field_name'])
      ->referencedEntities()[$plugin_parts['plugin_field_delta']];
  }

  /**
   * Explode the plugin id and extract information about the plugin.
   *
   * @param string $plugin_id
   *
   * @return array
   */
  private function getPluginParts(string $plugin_id): array {
    $parts = explode(':', $plugin_id);
    if (count($parts) < 4) {
      return [];
    }
    return [
      'plugin_type_id' => $parts[0],
      'plugin_entity_type_id' => $parts[1],
      'plugin_field_name' => $parts[2],
      'plugin_field_delta' => $parts[3],
      'plugin_field_bundle' => $parts[4],
    ];
  }

  /**
   * Returns the plugin's paragraph title.
   *
   * @param string $plugin_id
   *   The plugin id.
   *
   * @return string
   *   The paragraph title.
   */
  public function getTitle(string $plugin_id): string {
    $paragraph = $this->getParagraph($plugin_id);

    if (empty($paragraph)) {
      return '';
    }

    // Use admin title as block label.
    if ($paragraph->hasAdminTitle()) {
      return $paragraph->getAdminTitle();
    }

    // Fallback to paragraph summary title behavior.
    return $paragraph->getSummary();
  }

  /**
   * Set add/edit form title and display properties.
   *
   * @param array $form
   *   The form values.
   * @param string $title
   *   The title.
   */
  public function setFormTitle(array &$form, string $title) {
    $section = &$form['flipper']['front']['settings'];
    $section['admin_label']['#type'] = 'hidden';
    $section['label']['#type'] = 'hidden';
    $section['label']['#required'] = FALSE;
    $section['label_display']['#type'] = 'hidden';
    $section['label_display']['#value'] = 0;
    $section['label']['#value'] = $title;
  }

}

==> paragraph_blocks-4.1.2.tar.gz: paragraph_blocks/src/ParagraphBlocksLabeller.php <==
<?php

namespace Drupal\paragraph_blocks;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\paragraph_blocks\Entity\ParagraphBlocksEntity;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\paragraphs_library\Entity\LibraryItem;

/**
 * Labels the paragraph blocks once the entity context is known.
 */
class ParagraphBlocksLabeller {

  use StringTranslationTrait;

  /**
   * The plugin type id.
   *
   * @var string
   */
  const PLUGIN_TYPE_ID = 'paragraph_field';

  /**
   * The label format.
   *
   * @var string
   */
  const LABEL_FORMAT = 'Page: @label';

  /**
   * The current entity, or NULL.
   *
   * @var \Drupal\Core\Entity\EntityInterface|null
   */
  protected ?EntityInterface $entity;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The paragraph blocks entity manager.
   *
   * @var \Drupal\paragraph_blocks\ParagraphBlocksEntityManager
   */
  protected ParagraphBlocksEntityManager $paragraphBlocksEntityManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * Paragraphs entity storage instance.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface $paragraphStorage
   */
  private EntityStorageInterface $paragraphStorage;

  /**
   * Constructs a new ParagraphBlocksLabeller object.
   *
   * @param \Drupal\paragraph_blocks\ParagraphBlocksEntityManager $paragraph_blocks_entity_manager
   *   The Paragraph blocks entity manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The interface for an entity field manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(
    ParagraphBlocksEntityManager $paragraph_blocks_entity_manager,
    EntityFieldManagerInterface $entity_field_manager,
    ModuleHandlerInterface $module_handler,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->paragraphBlocksEntityManager = $paragraph_blocks_entity_manager;
    $this->entity = $this->paragraphBlocksEntityManager->getRefererEntity();
    $this->entityFieldManager = $entity_field_manager;
    $this->moduleHandler = $module_handler;
    $this->paragraphStorage = $entity_type_manager->getStorage('paragraph');
  }

  /**
   * Removed unused paragraphs and update the layout builder title.
   *
   * @param array $definitions
   *   The plugin definitions.
   */
  public function hookLayoutBuilderChooseBlocksAlter(array &$definitions) {
    // Loop through all the plugin definitions.
    foreach ($definitions as $plugin_id => $definition) {
      if ($this->isParagraphField($plugin_id)) {
        // Remove if this paragraph field is not enabled or if there is no paragraph data for the delta.
        if (!$this->paragraphFieldIsEnabled($plugin_id) || is_null($this->getParagraph($plugin_id))) {
          unset($definitions[$plugin_id]);
        }
        // Replace the admin label.
        else {
          $paragraph = $this->getParagraph($plugin_id);
          // Replace the paragraph if it is from the paragraphs library.
          if ($this->isParagraphFromLibrary($paragraph)) {
            $paragraph = $this->getParagraphFromLibrary($paragraph);
            $definitions[$plugin_id]['category'] .= ' ' . $this->t('from library');
          }
          $definitions[$plugin_id]['admin_label'] = $this->getTitle($paragraph);
        }
      }
    }
  }

  /**
   * Check if the given plugin id represents a paragraph field.
   *
   * @param string $plugin_id
   *
   * @return bool
   */
  public function isParagraphField(string $plugin_id): bool {
    $plugin_parts = $this->getPluginInfo($plugin_id);
    return $plugin_parts['plugin_type_id'] === self::PLUGIN_TYPE_ID;
  }

  /**
   * Check if this paragraph is from the paragraphs library.
   *
   * This is only relevant if the paragraphs_library module is enabled.
   *
   * @param \Drupal\paragraph_blocks\Entity\ParagraphBlocksEntity $paragraph
   *
   * @return bool
   */
  public function isParagraphFromLibrary(ParagraphBlocksEntity $paragraph): bool {
    return $paragraph->bundle() === 'from_library';
  }

  /**
   * Check if paragraph field is enabled.
   *
   * Only check the field bundle if it exists. This is new to the 2.x branch.
   * So this check exists for backwards compatability with plugins saved using
   * the 1.x branch.
   *
   * @param string $plugin_id
   *
   * @return bool
   */
  public function paragraphFieldIsEnabled(string $plugin_id): bool {
    $plugin_parts = $this->getPluginInfo($plugin_id);
    if (!empty($plugin_parts) && isset($plugin_parts['plugin_field_bundle'])) {
      $field_definitions = $this->entityFieldManager
        ->getFieldDefinitions($plugin_parts['plugin_entity_type_id'], $plugin_parts['plugin_field_bundle']);
      $field_config = $field_definitions[$plugin_parts['plugin_field_name']]
        ->getConfig($plugin_parts['plugin_field_bundle']);
      if ($field_config->getThirdPartySetting('paragraph_blocks', 'status', TRUE)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Returns the plugin's paragraph.
   *
   * @param string $plugin_id
   *   The plugin id.
   *
   * @return ?\Drupal\paragraph_blocks\Entity\ParagraphBlocksEntity
   *   The paragraph title.
   */
  public function getParagraph(string $plugin_id): ?ParagraphBlocksEntity {
    $plugin_parts = $this->getPluginInfo($plugin_id);

    // Return NULL if this plugin id represents no valid paragraph data.
    if (!$this->entity
      || $this->entity->bundle() !== $plugin_parts['plugin_field_bundle']
      || $plugin_parts['plugin_field_delta'] >= $this->entity->get($plugin_parts['plugin_field_name'])
        ->count()
    ) {
      return NULL;
    }

    // Get the referenced paragraph.
    return $this->entity->get($plugin_parts['plugin_field_name'])
      ->referencedEntities()[$plugin_parts['plugin_field_delta']];
  }

  /**
   * Gets the original paragraph referenced by the paragraphs library item.
   *
   * @param \Drupal\paragraphs\Entity\Paragraph $paragraph
   *   Paragraphs library entity.
   *
   * @return ?\Drupal\Core\Entity\EntityInterface
   */
  public function getParagraphFromLibrary(Paragraph $paragraph): ?EntityInterface {
    if ($paragraph->hasField('field_reusable_paragraph')) {
      $library_item = LibraryItem::load($paragraph->get('field_reusable_paragraph')->target_id);
      return $this->paragraphStorage->loadRevision($library_item->get('paragraphs')->target_revision_id);
    }
    return NULL;
  }

  /**
   * Explode the plugin id and extract information about the plugin.
   *
   * @param string $plugin_id
   *
   * @return array
   */
  public function getPluginInfo(string $plugin_id): array {
    $parts = explode(':', $plugin_id);
    $count = count($parts);
    if ($count < 4) {
      return [
        'plugin_type_id' => '',
        'plugin_entity_type_id' => '',
        'plugin_field_name' => '',
        'plugin_field_delta' => '',
        'plugin_field_bundle' => '',
        'count' => $count,
      ];
    }
    return [
      'plugin_type_id' => $parts[0] ?? '',
      'plugin_entity_type_id' => $parts[1] ?? '',
      'plugin_field_name' => $parts[2] ?? '',
      'plugin_field_delta' => $parts[3] ?? '',
      'plugin_field_bundle' => $parts[4] ?? '',
      'count' => $count,
    ];
  }

  /**
   * Returns the plugin's paragraph title.
   *
   * @param \Drupal\paragraph_blocks\Entity\ParagraphBlocksEntity $paragraph
   *   The paragraph block.
   *
   * @return string
   *   The paragraph title.
   */
  public function getTitle(ParagraphBlocksEntity $paragraph): string {
    // Use admin title as block label.
    if ($paragraph->hasAdminTitle()) {
      return $paragraph->getAdminTitle();
    }

    // Fallback to paragraph summary title behavior.
    return $paragraph->getSummary();
  }

  /**
   * Set add/edit form title and display properties.
   *
   * @param array $form
   *   The form values.
   * @param string $title
   *   The title.
   */
  public function setFormTitle(array &$form, string $title) {
    $section = &$form['flipper']['front']['settings'];
    $section['admin_label']['#type'] = 'hidden';
    $section['label']['#type'] = 'hidden';
    $section['label']['#required'] = FALSE;
    $section['label_display']['#type'] = 'hidden';
    $section['label_display']['#value'] = 0;
    $section['label']['#value'] = $title;
  }

}
